def sudoku_solve_milp(S):
    r"""
    Solve the Sudoku problem with Mixed Ingeger Linear Programming (MILP).

    ALGORITHM:

    For each position p and each number n, we put a boolean variable saying
    whether the number at position (i,j) is n. Then we add constraints about:

        - The numbers appearing in the incomplete grid are forced
        - each position has exactly one number
        - The fact that each numbers appears:
            - in each line
            - in each column
            - in each square block

    EXAMPLES:

    ::

        sage: S = Sudoku('1....7.9..3..2...8..96..5....53..9...1..8...26....4...3......1..4......7..7...3..') ; S
        +-----+-----+-----+
        |1    |    7|  9  |
        |  3  |  2  |    8|
        |    9|6    |5    |
        +-----+-----+-----+
        |    5|3    |9    |
        |  1  |  8  |    2|
        |6    |    4|     |
        +-----+-----+-----+
        |3    |     |  1  |
        |  4  |     |    7|
        |    7|     |3    |
        +-----+-----+-----+

        sage: sudoku_solve_milp(S)
        +-----+-----+-----+
        |1 6 2|8 5 7|4 9 3|
        |5 3 4|1 2 9|6 7 8|
        |7 8 9|6 4 3|5 2 1|
        +-----+-----+-----+
        |4 7 5|3 1 2|9 8 6|
        |9 1 3|5 8 6|7 4 2|
        |6 2 8|7 9 4|1 3 5|
        +-----+-----+-----+
        |3 5 6|4 7 8|2 1 9|
        |2 4 1|9 3 5|8 6 7|
        |8 9 7|2 6 1|3 5 4|
        +-----+-----+-----+

    ::

        sage: S = Sudoku('5...8..49...5...3..673....115..........2.8..........187....415..3...2...49..5...3') ; S
        +-----+-----+-----+
        |5    |  8  |  4 9|
        |     |5    |  3  |
        |  6 7|3    |    1|
        +-----+-----+-----+
        |1 5  |     |     |
        |     |2   8|     |
        |     |     |  1 8|
        +-----+-----+-----+
        |7    |    4|1 5  |
        |  3  |    2|     |
        |4 9  |  5  |    3|
        +-----+-----+-----+

        sage: sudoku_solve_milp(S)
        +-----+-----+-----+
        |5 1 3|6 8 7|2 4 9|
        |8 4 9|5 2 1|6 3 7|
        |2 6 7|3 4 9|5 8 1|
        +-----+-----+-----+
        |1 5 8|4 6 3|9 7 2|
        |9 7 4|2 1 8|3 6 5|
        |3 2 6|7 9 5|4 1 8|
        +-----+-----+-----+
        |7 8 2|9 3 4|1 5 6|
        |6 3 5|1 7 2|8 9 4|
        |4 9 1|8 5 6|7 2 3|
        +-----+-----+-----+

    ::

    Some other examples, to test:

    ::

        sage: S = Sudoku('..............3.85..1.2.......5.7.....4...1...9.......5......73..2.1........4...9') ; S
        +-----+-----+-----+
        |     |     |     |
        |     |    3|  8 5|
        |    1|  2  |     |
        +-----+-----+-----+
        |     |5   7|     |
        |    4|     |1    |
        |  9  |     |     |
        +-----+-----+-----+
        |5    |     |  7 3|
        |    2|  1  |     |
        |     |  4  |    9|
        +-----+-----+-----+

        sage: sudoku_solve_milp(S)
        +-----+-----+-----+
        |9 8 7|6 5 4|3 2 1|
        |2 4 6|1 7 3|9 8 5|
        |3 5 1|9 2 8|7 4 6|
        +-----+-----+-----+
        |1 2 8|5 3 7|6 9 4|
        |6 3 4|8 9 2|1 5 7|
        |7 9 5|4 6 1|8 3 2|
        +-----+-----+-----+
        |5 1 9|2 8 6|4 7 3|
        |4 7 2|3 1 9|5 6 8|
        |8 6 3|7 4 5|2 1 9|
        +-----+-----+-----+

    A larger one:

    ::

        sage: S = Sudoku('....a..69.3....1d.2...8....e.4....b....5..c.......7.......g...f....1.e..2.b.8..3.......4.d.....6.........f..7.g..9.a..c...5.....8..f.....1..e.79.c....b.....2...6.....g.7......84....3.d..a.5....5...7..e...ca.....3.1.......b......f....4...d..e..g.92.6..8....') ; S
        +-----------+-----------+-----------+-----------+
        |           |10        6| 9     3   |          1|
        |13     2   |       8   |         14|    4      |
        |      11   |          5|      12   |           |
        |       7   |           |      16   |      15   |
        +-----------+-----------+-----------+-----------+
        |          1|   14      | 2    11   | 8        3|
        |           |          4|   13      |          6|
        |           |           |   15      | 7    16   |
        |    9    10|      12   |       5   |           |
        +-----------+-----------+-----------+-----------+
        | 8       15|           |    1      |14     7  9|
        |   12      |      11   |           | 2         |
        | 6         |      16   | 7         |          8|
        | 4         |    3    13|      10   | 5         |
        +-----------+-----------+-----------+-----------+
        |    5      |    7      |14         |12 10      |
        |          3|    1      |           |   11      |
        |           |15         |    4      |   13      |
        |14       16|    9  2   | 6        8|           |
        +-----------+-----------+-----------+-----------+

        sage: sudoku_solve_milp(S)
        +-----------+-----------+-----------+-----------+
        |16 15 14 12|10  2  4  6| 9  8  3 13|11  7  5  1|
        |13  3  2  5| 7 11  8 15| 1 10  6 14|16  4  9 12|
        |10  8 11  9|14 16  1  5| 4  7 12 15|13  3  6  2|
        | 1  6  7  4|13 12  3  9|11  5 16  2|10  8 15 14|
        +-----------+-----------+-----------+-----------+
        | 5 13 12  1|16 14 15  7| 2  6 11  4| 8  9 10  3|
        | 3 14 16  7| 1  5 10  4| 8 13  9 12|15  2 11  6|
        | 2  4  6  8| 3 13  9 11|10 15 14  1| 7 12 16  5|
        |11  9 15 10| 6  8 12  2|16  3  5  7| 4 14  1 13|
        +-----------+-----------+-----------+-----------+
        | 8 16 13 15|12  4  5 10| 3  1  2 11|14  6  7  9|
        | 7 12  3 14| 9  6 11  1|13 16  8  5| 2 15  4 10|
        | 6 10  5 11| 2 15 16 14| 7 12  4  9| 3  1 13  8|
        | 4  1  9  2| 8  3  7 13|15 14 10  6| 5 16 12 11|
        +-----------+-----------+-----------+-----------+
        | 9  5  1 13|11  7  6 16|14  2 15  3|12 10  8  4|
        |15  2  4  3| 5  1 13  8|12  9  7 10| 6 11 14 16|
        |12 11  8  6|15 10 14  3| 5  4  1 16| 9 13  2  7|
        |14  7 10 16| 4  9  2 12| 6 11 13  8| 1  5  3 15|
        +-----------+-----------+-----------+-----------+

    AUTHOR:

    - Thierry Monteil <sage@lma.metelu.net>

    """
    n = S.n
    N = n^2
    p = MixedIntegerLinearProgram()
    p.set_objective(None)

    ## x[i][j][k] is 1 if the (i,j) coordinate has letter k
    ## 0 <= i,j <= N and 1 <= k <= N+1
    x = p.new_variable(binary=True)

    M = S.to_matrix()

    # values fixed by the initial matrix
    for line in range(N):
        for column in range(N):
            letter = M[line,column]
            if letter != 0:
                p.add_constraint(x[line, column, letter] == 1)

    # letters do not overlap
    for line in range(N):
        for column in range(N):
            p.add_constraint(sum(x[line, column, letter] for letter in range(1,N+1)) == 1)

    # line conditions
    for letter in range(1,N+1):
        for line in range(N):
            p.add_constraint(sum(x[line, column, letter] for column in range(N)) == 1)

    # column conditions
    for letter in range(1,N+1):
        for column in range(N):
            p.add_constraint(sum(x[line, column, letter] for line in range(N)) == 1)

    # block conditions
    for letter in range(1,N+1):
        for l_block in range(n):
            for c_block in range(n):
                p.add_constraint(sum(x[n*l_block+l, n*c_block+c, letter] for l in range(n) for c in range(n)) == 1)

    p.solve()

    dico = p.get_values(x)
    Mat = matrix(N,N)
    for line in range(N):
        for column in range(N):
            for letter in range(1,N+1):
                if dico[line, column, letter] == 1:
                    Mat[line,column] = letter

    return Sudoku(Mat)

